({{ component_type }} "Annotation" 
{% if version >= 6 %}
(version {{ timestamp }}) (generator pinout)
{% endif %}
  (layer "{{ layer }}")
{% if version >= 6 %}
  (tedit 0)
{% endif %}
  (descr "Add an annotation")
(attr {% if version >= 6 %}exclude_from_pos_files exclude_from_bom{%else%}virtual{%endif%})
  (fp_text reference "pinout_annotation" (at 9.525 6.985) (layer "{{ layer }}") {% if annotation.hide_fp_text_reference %}hide{% endif %} 
    (effects (font (size 1 1) (thickness 0.15)) (justify left))
  )
  (fp_text value "Annotation_content_here" (at 23.495 6.985 unlocked) (layer "{{ layer }}")
    (effects (font (size 1 1) (thickness 0.15)) (justify left))
  )
  (fp_text user "vh" (at 9.525 8.89) (layer "{{ layer }}")  {% if annotation.hide_fp_text_user %}hide{% endif %} 
    (effects (font (size 1 1) (thickness 0.15)) (justify left))
  )
  (fp_line (start 1 0) (end -1 0) (layer "{{ layer }}") (width 0.12))
  (fp_line (start 0 -1) (end 0 1) (layer "{{ layer }}") (width 0.12))
  (fp_circle (center 0 0) (end 0 -0.635) (layer "{{ layer }}") (width 0.12){% if version >= 6 %} (fill none){% endif %} )
)
